;;;;;;;;;;;;;;;;;;;;;
; Setup patch and breed variables 
;;;;;;;;;;;;;;;;;;;;;
patches-own [ detritus  food  penzyme  lost-enzyme  proot ]          ; Spent-enzyme is that used to make food.  Lost-enzyme is that that decays or mineralizes

breed [ microbes microbe ]
breed [ predators predator ]
breed [ roots root ]
microbes-own [ energy  consumed  respired  tick-consumed  tick-respired ]                                       ; Spent-enzmye, lost-enzyme (above), consumed and respired are stored by not printed or plotted at the moment.  Accumulation variables.
predators-own [ energy  consumed  respired  tick-consumed  tick-respired ] 
  
globals [ lastmicro  lastpred  nowmicro  nowpred  total-detritus-added  per-patch-detritus  clumpiness    
    recent-microbes              
    recent-predators             
    recent-microbe-respired      
    recent-predator-respired     
    recent-enzyme                
    recent-detritus              
    recent-food                  
    
    total-microbes                 ; This will not be a total of those that ever lived, but rather a count of "microbe-ticks".  The same for predators
    total-microbes-respired
    total-predators
    total-predators-respired
   ]  
  
;;;;;;;;;;;;;;;;;;;;;
; SETUP
;;;;;;;;;;;;;;;;;;;;;
to setup
  clear-all
  ; Initializing the lists to store three values.  If they were not initialized, errors occurred below.  That is why the initialization below starts from 3.

  setup-patches
  setup-microbes
  setup-predators
  set clumpiness -99
  if ( Detritus-model = "Root, Continuous" or Detritus-model = "Root, Periodic" ) 
  [ 
     setup-roots
   ;  To spot check clump calcs: ask patches [  if pycor < 50 [ set proot 1  set pcolor brown ] ]
   ;  To spot check clump calcs: ask patches [ if pycor < 99 [ set proot 1 set pcolor brown ] ] 
   ;  To spot check clump calcs: ask patches [ if pycor mod 2 = 0 [ set proot 1  set pcolor brown ] ]
   ;  To spot check clump calcs: ask patch 50 50 [ set proot 1  set pcolor brown ]    ask patch 50 51 [ set proot 1  set pcolor brown ] 
   ;  To spot check clump calcs: ask n-of 8000 patches [ set proot 1  set pcolor brown ]
   ;  To spot check clump calcs:    
   ;    ask patches [ 
   ;      ifelse pycor mod 2 = 0 
   ;      [ if pxcor mod 2 = 0  [ set proot 1  set pcolor brown ] ] 
   ;      [ if ( pxcor + 1 ) mod 2 = 0 [ set proot 1 set pcolor brown ] ] 
   ;    ]  
    get-clumpiness
  ]
  set total-microbes 0
  set total-microbes-respired 0  
  set total-predators 0
  set total-predators-respired 0

  reset-ticks
end

;;;;;;;;;;;;;;;;;;;;;
; SETUP-PATCHES
;;;;;;;;;;;;;;;;;;;;;
to setup-patches
  ask patches [ 
    if ( not ( Detritus-model = "Root, Continuous"  or Detritus-model = "Root, Periodic" ) )
    [ set detritus init-detritus ]
    set food 0
    set penzyme 0
    set pcolor rgb detritus food penzyme                        ; Paint each pixel using three color guns, red for detritus, green for food, and blue for enzymes
    set proot 0
  ] 
end

;;;;;;;;;;;;;;;;;;;;;
; SETUP-MICROBES
;;;;;;;;;;;;;;;;;;;;;
to setup-microbes
  create-microbes init-microbes [
    set shape "circle"
    set color white
    set size 0.6
    setxy random-xcor random-ycor
    set energy init-energy
    set consumed 0.
    set respired 0.
  ]
end

;;;;;;;;;;;;;;;;;;;;;
; SETUP-PREDATORS
;;;;;;;;;;;;;;;;;;;;;
to setup-predators
  create-predators init-predators [
    set shape "circle"
    set color yellow
    set size 1.2
    setxy random-xcor random-ycor
    set energy init-predator-energy
    set consumed 0.
    set respired 0.
  ]
end

;;;;;;;;;;;;;;;;;;;;;
; SETUP-ROOTS
;;;;;;;;;;;;;;;;;;;;;
to setup-roots
  let root-length 0      ; total number of patches labeled as root
  create-roots init-roots 
  [  set size 1.0
     
     set color brown
     setxy random-xcor random-ycor
     set heading random 360
     set pcolor green
     set pen-size 2.5
     pen-down     
  ]
  ; Keep growing roots until they're the full length and all die
  set root-length count patches with [ proot = 1 ]
  while [ root-length < total-root-length ] 
  [
    ask one-of roots
    [ 
      set pcolor green
      right random ( 2 * max-turn ) - max-turn
      ; Allow the roots to cross the torus, but include a death rate to limit growth
      forward 1
      set proot 1
      set detritus init-detritus
      if ( random-float 1.0 < bifurcate-prob ) [
        hatch 1 
        [
          ; Assign a turn angle for the new branch that uses a normal curve from a mean.  It either turns left or right, depending upon a draw.
          ifelse ( random-float 1.0 < 0.5 ) 
          [ right ( random-normal branch-angle-mean branch-angle-std ) ]
          [ left  ( random-normal branch-angle-mean branch-angle-std ) ]          
        ]
      ]
    ]
    set root-length count patches with [ proot = 1 ] 
  ]
end


;;;;;;;;;;;;;;;;;;;;;
;********************
; GO
;********************
;;;;;;;;;;;;;;;;;;;;;
to go
  ; Do phase calculations
  let j 0
  set lastmicro nowmicro
  set lastpred nowpred
  move
  move-predators
  produce-enzyme
  diffuse-enzyme
  convert-to-food
  feed
  feed-predators
  mortality
  mortality-predators
  reproduce
  reproduce-predators
  decay-enzyme
  if ( Detritus-model = "Rain, Continuous" ) 
    [ detritus-rain ]
  if ( Detritus-model = "Rain, Periodic" )
    [ if ( ticks mod period = 0 and ticks > 1 )
      [ detritus-period ]
    ]
  if ( Detritus-model = "Root, Continuous" )
    [ 
      detritus-root-continuous     
    ]
  if ( Detritus-model = "Root, Periodic" )
    [ if ( ticks mod period = 0 and ticks > 1 )
      [ detritus-root-periodic ]
    ]
  diffuse detritus detritus-diffusion-rate   
  set nowmicro count microbes
  set nowpred count predators

  ; Save values for long term summaries.   This must be done because if agents in a agentset all die, no statistics can be done
  set total-microbes   total-microbes + count microbes
  set total-predators  total-predators + count predators
  set total-microbes-respired   total-microbes-respired + sum [ tick-respired ] of microbes
  set total-predators-respired  total-predators-respired + sum [ tick-respired ] of predators
  
  let v 0.0
  ; The following should be 1 rather than 2, but it doesn't create a list and generates an error.  I will duplicate the first entry.  It shouldn't change the mean of 100 entries by more
  ; a very small amount, and will disappear after 500 ticks.
  ifelse ( ticks <= 1 ) 
   [
     set recent-microbes ( list count microbes ) 
     set recent-predators ( list count predators ) 
     set recent-microbe-respired ( list sum [ tick-respired ] of microbes ) 
     set recent-predator-respired ( list sum [ tick-respired ] of predators ) 
     set recent-enzyme ( list mean [ penzyme ] of patches ) 
     set recent-detritus ( list mean [ detritus ] of patches ) 
     set recent-food ( list mean [ food ] of patches ) 
   ]
   [
     ifelse ( ticks <= 500 )
     [
       set recent-microbes lput count microbes recent-microbes
       set recent-predators lput count predators recent-predators
       set recent-microbe-respired lput sum [ tick-respired ] of microbes recent-microbe-respired
       set recent-predator-respired lput sum [ tick-respired ] of predators recent-predator-respired
       set recent-enzyme lput mean [ penzyme ] of patches recent-enzyme
       set recent-detritus lput mean [ detritus ] of patches recent-detritus
       set recent-food lput mean [ food ] of patches recent-food
     ]
     [
       set j ticks mod 500 
       set recent-microbes  replace-item j  recent-microbes  count microbes
       set recent-predators  replace-item j  recent-predators  count predators
       set recent-microbe-respired  replace-item j  recent-microbe-respired  sum [ tick-respired ] of microbes
       set recent-predator-respired  replace-item j  recent-predator-respired  sum [ tick-respired ] of predators
       set recent-enzyme  replace-item j  recent-enzyme  mean [ penzyme ] of patches
       set recent-detritus  replace-item j  recent-detritus  mean [ detritus ] of patches
       set recent-food  replace-item j  recent-food  mean [ food ] of patches
     ]
   ]

  ifelse ( hide-agents = true )
   [ ask turtles [ hide-turtle ] ]
   [ ask turtles [ show-turtle ] ]
  draw-patches
;  if count predators = 0 [ stop ]

  tick
end

;;;;;;;;;;;;;;;;;;;;;
; MOVE
;;;;;;;;;;;;;;;;;;;;;
to move
  ask microbes [
    ; Using random turning angles right now, Brownian
    right random 360
    forward microbe-speed
    set energy  ( energy - move-energy-cost )
  ]
end

;;;;;;;;;;;;;;;;;;;;;
; MOVE-PREDATORS
;;;;;;;;;;;;;;;;;;;;;
to move-predators
  ask predators [
    ; Using random turning angles right now, Brownian
    right random 360
    forward predator-speed
    set energy  ( energy - move-predator-energy-cost )
  ]
end

;;;;;;;;;;;;;;;;;;;;;
; PRODUCE-ENZYME
;;;;;;;;;;;;;;;;;;;;;
to produce-enzyme
  ask microbes [
    ; Going to put a small buffer in for enzyme cost, so that the microbe doesn't use its last bit of energy to make enzyme.  This will be hardwired.
    if penzyme < enzyme-threshold and energy > ( enzyme-cost * enzyme-energy-buffer ) [
      set energy  ( energy - enzyme-cost )       
      set penzyme  ( penzyme + 1.0 )                    ; Enzymes are generated in one unit increments
    ]
  ]
end

;;;;;;;;;;;;;;;;;;;;;
; DIFFUSE-ENZYME
;;;;;;;;;;;;;;;;;;;;;
to diffuse-enzyme
  diffuse penzyme diffusion-rate  
end

;;;;;;;;;;;;;;;;;;;;;
; DECAY-ENZYME
;;;;;;;;;;;;;;;;;;;;;
to decay-enzyme
  let decay-rate  ( 0.50 / decay-half-life )

  ask patches [ 
    set lost-enzyme  ( lost-enzyme + ( penzyme * decay-rate ) )
    set penzyme  ( penzyme * ( 1.0 - decay-rate ) )
  ]  
end

;;;;;;;;;;;;;;;;;;;;;
; CONVERT-TO-FOOD
;;;;;;;;;;;;;;;;;;;;;
to convert-to-food
  ask patches [
    if detritus > 0 and penzyme > 0 [
      let enzyme-cnt ( enzyme-efficiency * (  vMax * detritus ) / ( beta + detritus ) ) * penzyme
      while [ enzyme-cnt > 0 ] [
        let food-cnt food-per-unit-enzyme
        while [ detritus > 0 and food-cnt > 0 ] [
          set food  ( food + 1 )
          set food-cnt  ( food-cnt - 1 )
          set detritus  ( detritus - 1 )
          if ( detritus < 0 ) [ set detritus 0 ]
        ]
        
        set enzyme-cnt  ( enzyme-cnt - 1 )
      ]
  ;    set penzyme  ( penzyme - 1 )              NO ... M. Wallenstein said that enzymes wouldn't be 'consumed' during a conversion.    
  ;    set spent-enzyme  ( spent-enzyme + 1 )
    ]
  ]
end

;;;;;;;;;;;;;;;;;;;;;
; FEED
;;;;;;;;;;;;;;;;;;;;;
to feed
  ask microbes [
    let max-food  food
    set tick-consumed 0.0
    set tick-respired 0.0
    if ( max-food > 0 ) [
      if ( max-food > max-food-taken ) [ set max-food  max-food-taken ]
      let food-energy  ( max-food * food-energy-value )
      set energy  energy + ( food-energy * ( 1.0 - prop-respired ) ) 
      set food    food - max-food
      set consumed  ( consumed + ( food-energy * ( 1.0 - prop-respired ) ) )
      set respired  ( respired + ( food-energy * prop-respired ) )
      set tick-consumed  ( tick-consumed + ( food-energy * ( 1.0 - prop-respired ) ) )
      set tick-respired  ( tick-respired + ( food-energy * prop-respired ) )      
    ]
  ]
end


;;;;;;;;;;;;;;;;;;;;;
; FEED-PREDATORS
;;;;;;;;;;;;;;;;;;;;;
to feed-predators
  ask predators [
    let microbes-in-patch  microbes-here
    set tick-consumed 0.0
    set tick-respired 0.0
    if ( count microbes-in-patch >= max-predator-feed-rate ) 
      [ set microbes-in-patch n-of max-predator-feed-rate microbes-in-patch ]
    if ( count microbes-in-patch > 0 )
      [ 
        set energy  energy + ( ( count microbes-in-patch * mean [ energy ] of microbes-in-patch ) * ( 1.0 - pred-prop-respired ) )
        set consumed  consumed + ( ( count microbes-in-patch * mean [ energy ] of microbes-in-patch ) * ( 1.0 - pred-prop-respired ) )
        set respired  respired + ( ( count microbes-in-patch * mean [ energy ] of microbes-in-patch ) * pred-prop-respired )
        set tick-consumed  ( tick-consumed + ( count microbes-in-patch * mean [ energy ] of microbes-in-patch ) * ( 1.0 - pred-prop-respired ) )
        set tick-respired  ( tick-respired + ( count microbes-in-patch * mean [ energy ] of microbes-in-patch ) * pred-prop-respired )      
        ask microbes-in-patch [ die ]              ; Don't go into detritus.  Including respiration ... it goes off as CO2 etc.
      ]
  ]
end

;;;;;;;;;;;;;;;;;;;;;
; MORTALITY
;;;;;;;;;;;;;;;;;;;;;
to mortality
  ask microbes [
    if ( energy < 0 )
    [ 
      ; Can't use energy as a basis to add back to detritus ... the energy is 0, hence death.   Another slider ...
      set  detritus  ( detritus + microbe-to-detritus )  
      die 
    ]
  ]
end

;;;;;;;;;;;;;;;;;;;;;
; MORTALITY-PREDATORS
;;;;;;;;;;;;;;;;;;;;;
to mortality-predators
  ask predators [
    if ( energy < 0 )
    [
      set  detritus  ( detritus + predator-to-detritus ) 
      die 
    ]
  ]
end

;;;;;;;;;;;;;;;;;;;;;
; REPRODUCE
;;;;;;;;;;;;;;;;;;;;;
to reproduce
  ask microbes [
    if ( energy > energy-to-divide )
    [
      set energy  ( energy / 2.0 )
      hatch 1                                 ; hatched microbe will inherit energy, which has been split between the two microbes   
    ]
  ]
end

;;;;;;;;;;;;;;;;;;;;;
; REPRODUCE-PREDATORS
;;;;;;;;;;;;;;;;;;;;;
to reproduce-predators
  ask predators [
    if ( energy > predator-energy-to-divide )
    [
      set energy  ( energy / 2.0 )
      hatch 1                                 ; hatched microbe will inherit energy, which has been split between the two microbes   
    ]
  ]
end

;;;;;;;;;;;;;;;;;;;;;
; DETRITUS-RAIN
;;;;;;;;;;;;;;;;;;;;;
to detritus-rain
  set per-patch-detritus  ( detritus-per-tick / detritus-patches )   ; Recalculated each time step, but no matter, nice to keep it all contained.
  ask n-of detritus-patches patches [
    set detritus  ( detritus + per-patch-detritus )
    set total-detritus-added  ( total-detritus-added + per-patch-detritus )
  ]
end

;;;;;;;;;;;;;;;;;;;;;
; DETRITUS-PERIOD
;;;;;;;;;;;;;;;;;;;;;
to detritus-period
  set per-patch-detritus  ( ( detritus-per-tick * period ) / detritus-patches )       ; Must multiply by period to includge sufficient detritus.  ERROR, fixed in OCT 02 version
  ask n-of detritus-patches patches [
    set detritus  ( detritus + per-patch-detritus )
    set total-detritus-added  ( total-detritus-added + per-patch-detritus )
  ]
end

;;;;;;;;;;;;;;;;;;;;;
; DETRITUS-ROOT-CONTINUOUS
;;;;;;;;;;;;;;;;;;;;;
to detritus-root-continuous
  set per-patch-detritus  ( detritus-per-tick / total-root-length )   ; Recalculated each time step, but no matter, nice to keep it all contained.
  ask patches 
  [
    if ( proot = 1 ) 
    [ 
      set detritus  ( detritus + per-patch-detritus ) 
      set total-detritus-added  ( total-detritus-added + per-patch-detritus )
    ]
  ]
end

;;;;;;;;;;;;;;;;;;;;;
; DETRITUS-ROOT-PERIODIC
;;;;;;;;;;;;;;;;;;;;;
to detritus-root-periodic
  set per-patch-detritus  ( ( detritus-per-tick * period ) / total-root-length )          ; Must multiply by period to includge sufficient detritus.  Changed Oct 02, but only orders, not the mathematics.  That was so that it is symetric with the rain version.
  ask patches 
  [
    if ( proot = 1 ) 
    [ 
      set detritus  ( detritus + per-patch-detritus ) 
      set total-detritus-added  ( total-detritus-added + per-patch-detritus )
    ]
  ]
end




;;;;;;;;;;;;;;;;;;;;;
; GET-CLUMPINESS
;;;;;;;;;;;;;;;;;;;;;
to get-clumpiness
  ; Using FRAGSTATS manual, page 130 and onward.  The clumpiness metric, C4
  let P1 ( count patches with [ proot = 1 ] / count patches )
  ; type " P1 = " print P1
  let g00 0
  let g01 0
  let g10 0   
  let g11 0
  
  ; I would prefer Queen's rule, if the logic holds up.
  ask patches [
    let pr  proot    
    ask neighbors4 [
    ;ask neighbors [
        if ( pr = 0 and proot = 0 ) [ set g00  g00 + 1 ]
        if ( pr = 0 and proot = 1 ) [ set g01  g01 + 1 ]
        if ( pr = 1 and proot = 0 ) [ set g10  g10 + 1 ]
        if ( pr = 1 and proot = 1 ) [ set g11  g11 + 1 ]              
    ]      
  ]

  let G1  ( g11 / ( g01 + g11 ) )
  if G1 >= P1               [  set clumpiness ( ( G1 - P1 ) / ( 1.0 - P1 ) ) ]
  if G1 < P1 and P1 >= 0.5  [  set clumpiness ( ( G1 - P1 ) / ( 1.0 - P1 ) ) ]
  if G1 < P1 and P1 < 0.5   [  set clumpiness ( ( P1 - G1 ) / ( -1.0 * P1 ) ) ]

end


;;;;;;;;;;;;;;;;;;;;;
; DRAW-PATCHES
;;;;;;;;;;;;;;;;;;;;;
to draw-patches
  ; Old way ...  set pcolor rgb detritus food penzyme                        ; Paint each pixel using three color guns, red for detritus, green for food, and blue for enzymes
  ; Stretching colors in the red, green, and blue color guns
  ; Need statistics prior to stretching
  let  detritus-val 0
  let  food-val 0
  let enzyme-val 0
  let  detritus-mean  mean [ detritus ] of patches
  let  food-mean       mean [ food ] of patches
  let  enzyme-mean     mean [ penzyme ] of patches
  let  detritus-std   standard-deviation [ detritus ] of patches + 0.0001
  let  food-std        standard-deviation [ food ] of patches + 0.0001
  let  enzyme-std      standard-deviation [ penzyme ] of patches + 0.0001
  let  detritus-range ( detritus-std * 4.0 ) + 0.001
  let  food-range      ( food-std * 4.0 ) + 0.001
  let  enzyme-range    ( enzyme-std * 4.0 ) + 0.001
  
  let  detritus-factor  detritus-mean - ( detritus-std * 2.0 )
  let  food-factor       food-mean - ( food-std * 2.0 )
  let  enzyme-factor     enzyme-mean - ( enzyme-std * 2.0 )
  
;  type " MEAN   " print detritus-mean   
;  type " STD    " print detritus-std
;  type " FACTOR " print detritus-factor
        
  ask patches [
    set detritus-val  int ( ( ( detritus - detritus-factor ) / detritus-range ) * 200.0 ) 
    set food-val       int ( ( ( food - food-factor ) / food-range ) * 200.0 )
    set enzyme-val     int ( ( ( penzyme - enzyme-factor ) / enzyme-range ) * 200.0 )
    if ( detritus-val < 0 )   [ set detritus-val   0 ] 
    if ( detritus-val > 200 ) [ set detritus-val  200 ] 
    if ( food-val < 0 )   [ set food-val  0 ] 
    if ( food-val > 200 )   [ set food-val  200 ] 
    if ( enzyme-val < 0 )   [ set enzyme-val  0 ] 
    if ( enzyme-val > 200 )   [ set enzyme-val  200 ] 
    
    if Drawing-type = "Composite" [ set pcolor  rgb detritus-val  food-val  enzyme-val ] 
    if Drawing-type = "Detritus" [ set pcolor  scale-color  red    detritus-val  0 200  ] ;   print mean [ detritus-val ] of patches ]
    if Drawing-type = "Food"      [ set pcolor  scale-color  green  food-val       0 200 ]
    if Drawing-type = "Enzyme"    [ set pcolor  scale-color  blue   enzyme-val     0 200 ]
  ]
  
end


;;;;;;;;;;;;;;;;;;;;;
; CLEAR-PHASE-PLOTS
;;;;;;;;;;;;;;;;;;;;;
to clear-phase-plots
  set-current-plot "Phase-1"
  auto-plot-off
  clear-plot
  auto-plot-on
  set-current-plot "Phase-2"
  clear-plot
  set-current-plot "Phase-3"
  clear-plot
;  set-current-plot "Phase-4"
;  set-current-plot "Phase-5"    
end
@#$#@#$#@
GRAPHICS-WINDOW
259
14
726
502
-1
-1
4.57
1
10
1
1
1
0
1
1
1
0
99
0
99
0
0
1
ticks
30.0

BUTTON
14
14
83
47
Setup
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
117
14
179
47
Go
go
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
188
14
251
47
Go
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SLIDER
1268
36
1506
69
init-detritus
init-detritus
1
200
33
1
1
particles
HORIZONTAL

SLIDER
1269
71
1506
104
init-microbes
init-microbes
0
1000
500
1
1
individuals
HORIZONTAL

SLIDER
1780
79
2032
112
microbe-speed
microbe-speed
0
1.0
0.2
0.001
1
patches
HORIZONTAL

SLIDER
1523
73
1759
106
enzyme-threshold
enzyme-threshold
0
100
100
1
1
energy unit
HORIZONTAL

SLIDER
1269
106
1506
139
init-energy
init-energy
0
100
5
1
1
energy unit
HORIZONTAL

SLIDER
1523
109
1760
142
enzyme-cost
enzyme-cost
0
1.0
0.1
0.01
1
energy unit
HORIZONTAL

SLIDER
1523
181
1761
214
diffusion-rate
diffusion-rate
0
2.0
0.25
0.001
1
proportion
HORIZONTAL

SLIDER
1779
36
2029
69
food-per-unit-enzyme
food-per-unit-enzyme
1
10
3
1
1
food units
HORIZONTAL

SLIDER
1523
218
1761
251
vMax
vMax
0
1.0
1
0.001
1
time
HORIZONTAL

SLIDER
1781
115
2032
148
move-energy-cost
move-energy-cost
0
0.5
0.06
0.01
1
energy unit
HORIZONTAL

SLIDER
1781
157
2032
190
food-energy-value
food-energy-value
0.001
0.8
0.3
0.001
1
energy units
HORIZONTAL

SLIDER
1522
36
1758
69
decay-half-life
decay-half-life
0
1000
1000
1
1
ticks
HORIZONTAL

SLIDER
1782
194
2032
227
max-food-taken
max-food-taken
1
100
10
1
1
food units
HORIZONTAL

SLIDER
1783
280
2033
313
energy-to-divide
energy-to-divide
0
100
8
1
1
energy units
HORIZONTAL

MONITOR
184
67
253
112
Microbes
count microbes
0
1
11

CHOOSER
14
390
173
435
Drawing-type
Drawing-type
"Composite" "Detritus" "Food" "Enzyme"
0

SLIDER
1521
413
1775
446
detritus-per-tick
detritus-per-tick
0
10000
1000
10
1
particles
HORIZONTAL

SLIDER
1522
451
1775
484
detritus-patches
detritus-patches
0
1000
600
1
1
patches
HORIZONTAL

SLIDER
1782
236
2033
269
prop-respired
prop-respired
0
1.0
0.5
0.001
1
proportion
HORIZONTAL

CHOOSER
14
184
173
229
Detritus-model
Detritus-model
"Root, Continuous" "Root, Periodic" "Rain, Continuous" "Rain, Periodic"
3

SLIDER
1522
488
1776
521
period
period
1
1000
200
1
1
ticks
HORIZONTAL

PLOT
736
251
1259
497
Concentrations
Ticks
Mean per patch
0.0
10.0
0.0
10.0
true
true
"" ""
PENS
"Detritus" 1.0 0 -14070903 true "" "plot mean [ detritus ] of patches"
"Food" 1.0 0 -2674135 true "" "plot mean [ food ] of patches"
"Enzyme" 1.0 0 -5825686 true "" "plot mean [ penzyme ] of patches"

PLOT
736
14
1259
247
Populations
Ticks
Count
0.0
10.0
0.0
10.0
true
true
"" ""
PENS
"Microbes" 1.0 0 -16777216 true "" "plot count microbes"
"Predators" 1.0 0 -5298144 true "" "plot count predators"

TEXTBOX
15
444
165
492
Red   - Detritus\nGreen - Food\nBlue  - Enzyme
12
0.0
1

SLIDER
1523
145
1761
178
enzyme-energy-buffer
enzyme-energy-buffer
0
10.0
5
0.1
1
energy unit
HORIZONTAL

SLIDER
2045
36
2343
69
init-predators
init-predators
0
100
0
1
1
individuals
HORIZONTAL

SLIDER
2046
72
2343
105
init-predator-energy
init-predator-energy
0
100
50
1
1
energy units
HORIZONTAL

SLIDER
2047
135
2344
168
predator-speed
predator-speed
0
3.00
1.1
0.01
1
patches
HORIZONTAL

SLIDER
2047
171
2345
204
move-predator-energy-cost
move-predator-energy-cost
0
2.0
0.22
0.01
1
energy units
HORIZONTAL

SLIDER
2048
218
2345
251
max-predator-feed-rate
max-predator-feed-rate
0
10
2
1
1
food units
HORIZONTAL

SLIDER
2048
292
2346
325
predator-energy-to-divide
predator-energy-to-divide
0
100
51
1
1
energy units
HORIZONTAL

MONITOR
185
116
252
161
Predators
count predators
0
1
11

SLIDER
1269
142
1507
175
init-roots
init-roots
1
2000
4
1
1
count
HORIZONTAL

SLIDER
1268
241
1506
274
max-turn
max-turn
0
100
33
1
1
angle
HORIZONTAL

SLIDER
1269
276
1506
309
bifurcate-prob
bifurcate-prob
0
0.1
0.01
0.001
1
probability
HORIZONTAL

SLIDER
1269
311
1507
344
branch-angle-mean
branch-angle-mean
0
120
45
1
1
angle
HORIZONTAL

SLIDER
1269
346
1508
379
branch-angle-std
branch-angle-std
1
100
20
1
1
angle
HORIZONTAL

SLIDER
1280
451
1520
484
total-root-length
total-root-length
1
5000
600
1
1
patches
HORIZONTAL

TEXTBOX
1274
17
1424
35
Initial values
12
0.0
1

TEXTBOX
1271
191
1501
226
Root variables\n(used if set in Detritus-model)
12
0.0
1

TEXTBOX
1522
351
1701
398
Detritus variables\n(different ones used dependent upon Detritus-model)
12
0.0
1

TEXTBOX
1528
15
1678
33
Enzyme variables
12
0.0
1

TEXTBOX
1782
17
1932
35
Microbe variables
12
0.0
1

TEXTBOX
2048
18
2198
36
Predator initial values\n
12
0.0
1

SLIDER
1522
532
1777
565
detritus-diffusion-rate
detritus-diffusion-rate
0.0
1.0
0.25
0.001
1
proportion
HORIZONTAL

TEXTBOX
2049
116
2199
134
Predator variables
12
0.0
1

SWITCH
15
128
172
161
hide-agents
hide-agents
1
1
-1000

SLIDER
2048
255
2345
288
pred-prop-respired
pred-prop-respired
0.0
1.0
0.5
0.001
1
proportion
HORIZONTAL

PLOT
141
515
487
861
Phase-1
Microbes
Predators
100000.0
10.0
10000.0
10.0
true
false
"" ""
PENS
"pen-0" 1.0 0 -16777216 true "" "plotxy count microbes  count predators"

BUTTON
15
513
130
546
Clear Phase Plots
clear-phase-plots
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

PLOT
500
515
846
861
Phase-2
Microbes (t-1)
Microbes (t)
10000.0
1.0
10000.0
1.0
true
false
"" ""
PENS
"default" 1.0 0 -16777216 true "" "plotxy lastmicro nowmicro"

PLOT
858
516
1202
862
Phase-3
Predators (t-1)
Predators (t)
10000.0
1.0
10000.0
1.0
true
false
"" ""
PENS
"default" 1.0 0 -16777216 true "" "plotxy lastpred nowpred"

MONITOR
186
179
252
224
Root cells
count patches with [ proot = 1 ]
0
1
11

MONITOR
13
235
173
280
Total detritus added
total-detritus-added
0
1
11

SLIDER
1522
572
1777
605
microbe-to-detritus
microbe-to-detritus
0
0.3
0.01
0.001
1
particles
HORIZONTAL

SLIDER
1522
610
1778
643
predator-to-detritus
predator-to-detritus
0
1.00
0.1
0.001
1
particles
HORIZONTAL

MONITOR
14
285
112
330
Root clumpiness
clumpiness
3
1
11

SLIDER
1523
255
1761
288
beta
beta
0
1.0
0.3
0.001
1
microbe unit
HORIZONTAL

SLIDER
1522
292
1760
325
enzyme-efficiency
enzyme-efficiency
0
1.0
0.5
0.001
1
unitless
HORIZONTAL

@#$#@#$#@
# Enzyme Model Description

The model description follows the ODD (Overview, Design concepts, Details) protocol for describing agent-based models (Grimm et al. 2010). 

## 1. Purpose

We seek to improve understanding of the roles enzyme production play in soil food webs.  Specifically, the importance of spatial heterogeneity in the production of enzymes is of inter-est.  Well-established methods of analyses use equation-based representations of interactions, often using ordinary differential equations.  In their typical use, those methods do not capture spatial variability in enzyme production or other interactions of soil biota.  We created an agent-based simulation of a simple food web that includes enzymatic activity.  Results from this spatial representation may be compared with the equation-based results to indicate the importance of spatial representations.

## 2. Entities, state variables, and scales	

The simulation represents a thin layer of a 1/10th cm2 section of soil.  The area is composed of 100 x 100 cells (10,000 cells), with the area a torus, such that cells on opposite edges of the area are considered neighbors.  Agents that move off one side of the area appear on the oppo-site side.  That is a common approach that eliminates edge effects.  Time steps that are simu-lated may be thought of as seconds, but remain loosely defined.

The soil is populated by microbes and each has an energy value (in energy units).  Two other state variables store the energy consumed and the portion of energy that went to respiration through the life of the microbe, but these accumulators play no role in system dynamics.  Predators may be placed in the system.  They have the same three variables, energy reflecting the condition of the predator and the accumulators storing consumed energy and energy lost to respiration.    

Each cell contains three quantities: 1) detritus particles, which may be acted upon by 2) en-zymes produced by microbes to yield quantities of 3) food.  Some cells may be designated as roots, depending upon the scenario simulated.  An accumulator in each cell is not active in soil processes and used for bookkeeping only, storing the quantity of enzyme lost to degradation.  Enzymes and food units are scaled relative to other model components, so that enzymes are in ‘enzyme units’ and food is in ‘food units.’

Detritus is influenced by global variables that, 1: describe how much detritus enters the envi-ronment each time step (units: particles); 2: the number of cells to which detritus is added each time step (units: cells); 3: the period between detritus additions (time step); and 4: the diffusion rate of the detritus through the environment (proportion).  Variables 1 and 4 are used in all simulations, 2 is used in simulations of detritus rainfall but not root exudates, and 3 is used when detritus is added to the system periodically.  

Microbes are influenced by the following global variables, 1: a speed (cells per time step); 2: an energy cost for each move made (energy units); 3: the value of a food item to the microbe (energy units); 4: the maximum rate detritus may be converted to food, known as vMax (time step); 5: the influence microbes on enzyme activity, beta (per microbe); 6: an enzymatic effi-ciency (unitless); 7: proportion of energy acquired that is put to respiration (energy units); 8: the energy needed for the microbe to divide (energy units); and 9: the contribution of a mi-crobe that dies to detritus (particles).

Predators have global variables analogous to microbes, including speed (1), movement cost (2), the value of a food item (a microbe) to the predator (3), proportion energy used for respi-ration(7), energy to divide (9), and contribution to detritus (9).  There is also a maximum rate of capture of microbes (microbes per time step).   Both microbes and predators store their current locations and internal identifiers as well.

Parameters controlling the production or influence of enzymes include, 1: a half-life for en-zymatic decay (units: time steps); 2: the enzymatic concentration in a cell, above which a mi-crobe will not produce more enzyme (enzyme units); 3: the cost to produce one enzyme (en-ergy units); 4: an energy buffer, below which microbes will not produce enzyme (energy units); 5: enzymatic diffusion rate (proportion); and 6: the food produced per unit enzyme, given sufficient substrate (food units).

## 3. Process overview and scheduling

The model is implemented in NetLogo version 5.0 using a discrete representation of time.  After initialization (see section 5), the following processes occur, with each process expanded upon in a discussion of the submodels used (see section 7) and the name of the process in pa-rentheses.  

Simulate time step	
	Store variables for use in phase space plots (i.e., values at t-1)
	Microbes move					(move)
	Predators move					(move-predators)
	Microbes produce enzymes			(produce-enzyme)
	Diffuse enzymes across area			(diffuse-enzyme)
	Convert detritus to food			(convert-to-food)
	Microbes feed					(feed)
	Predators feed					(feed-predators)
	Mortality of microbes if energy depleted 	(mortality)
	Mortality of predators if energy depleted 	(mortality-predators)
	Reproduce microbes				(reproduce)
	Reproduce predators				(reproduce-predators)
	Allow enzymes to decay				(decay-enzymes)	
	Add detritus using one of the following four:
		Rain detritus continuously		(detritus-rain)
		Rain detritus periodically		(detritus-period)
		Detritus from roots, continuously	(detritus-root-continuous)
		Detritus from roots, periodic		(detritus-root-periodic)
	Diffuse detritus throughout the area 		
	Store variables for phase space plots (i.e., values at t)	
	Store variables for long-term accumulators
	Update the user interface
	Stop if predators are extinct
End current time step and proceed with the next time step
  
## 4. Design concepts

_Basic principles:_  The simulation is constructed to allow users to explore aspects of enzyme production in microbes, in a two-dimensional unit of soil that includes predators.   Detritus is added to the soil in one of four combinations of two factors: continuous or periodic inflow, and distributed randomly or along roots.  The configurations of roots may be changed.  Mi-crobes move randomly across the soil, and produce enzymes in the presence of detritus.  The enzymes convert detritus to materials that the microbes may consume and gain energy (i.e., food).  Energy is spent by microbes through enzyme production, movement, and reproduction, and if energy drops to zero for a given microbe, it dies.  Predators are present in the soil in some simulations, and feed on microbes if they share the same location, up to some number of microbes that may be eaten.  Like the microbes, predators gain energy from the microbes they eat, use energy to move and to reproduce, and if an individual’s energy is depleted it dies.  Respiration of microbes and predators is accumulated during simulations.

The model represents a stylized soil patch, with parameters controlling dynamics set through references to known relationships where possible.  That said, the values or ranges of variables are sometimes not known.  The simulation is intended to provide an experimental platform to help understand how system attributes relate, rather than represent a particular soil and biotic community.  By adjusting parameters and rates, such as the rate of degradation of enzymes, rate of detritus inputs, energy required by microbes and predators to reproduce, and patterns of root formation, users may explore how the dynamics and spatial patterning of soil biota relate to primary system attributes.  

_Emergence:_  Two major emergent properties are of most interest, the population responses of microbes and predators and their interactions, and the spatial distribution of microbes, preda-tors, detritus, food, and enzymes.  The cyclic responses of biota are of interest, and especially the longevity of populations.  The stability of the system components in four patterns of detri-tus delivery are of particular interest: with roots and continuous detritus delivery, with roots and periodic detritus delivery, with rain in randomly selected cells in continuous delivery, and with rain in randomly selected cells in a periodic delivery of detritus.

_Adaptation, Objectives, and Learning:_  The organisms in the simulation do not adapt in the broadest sense in this application.  There is no capacity in this application for the heredity of traits from one generation to the next, beyond the trivial inheritance of an energy level of a new organism during fission.  Microbes may produce or not produce enzymes based the con-centration of enzymes in a given cell.  Microbes and predators have no inherent objectives.  They share feeding behavior and will reproduce and populate a space, but have no intrinsic objectives beyond these.  Learning is not part of the current simulation.

_Prediction:_ The organisms represented in the simulation do not make predictions in any direct sense.  Microbes do modify their environment for their future benefit by producing enzymes that convert detritus to food that may be consumed.  Microbes move while producing en-zymes, but the biased random walk they perform plus the diffusion applied to enzymes means that the individual producers of enzymes are likely to benefit from the investment.

_Sensing:_  Microbes and predators can sense the food and other resources that occur on the cell they occupy.  Specifically, microbes are aware of the presence of detritus and the concentra-tion of enzymes.  If the concentration of enzymes exceeds a threshold, microbes do not pro-duce more enzymes.  Microbes are aware of the presence of food, and predators are aware of the presence of microbes on their cell.  

_Interaction:_ Microbes compete for food resources, as do predators.  Predators feed on mi-crobes.  

_Stochasticity:_ Many aspects of the simulation include stochastic components.  The locations of roots are random and their growth during initialization includes random turning angles and a stochastic probability of branching for roots based on a normally distributed branching angle and a deviation indicated in a global variable.  The initial locations of microbes and predators are random (see Section 5).  

The modeling tool used randomizes the order in which agents and cells perform operations.  For microbes and predators, they turn a random heading prior to moving during a time step.  

When predators feed on in a cell that contains more than the maximum microbes that they can eat within a given time step, the microbes they do eat are selected randomly.  When detritus is added to the simulation without roots present, patches that receive detritus are selected ran-domly, each time step if using a continuous addition of detritus, or each period if that option is used.  

_Observation:_  Populations of microbes and predators, and average concentrations of detritus, enzymes, and food are stored from each simulation.  Simulation parameters are stored as well, such as the use of roots in modeling or not.  As simulations progress, plots show the numbers of microbes and predators, plus the average detritus, enzymes, and food available across the cells.  Another set of plots provide phase diagrams of microbes versus predators, microbes at the current and previous time step, and predators at the current and previous time step.   Cur-rent values are displayed for populations of microbes and predators and their average energy, the length of roots, and average detritus, enzymes, and food in cells.

The main outputs of each simulation were summarized over the long term, defined in the model as 500 time steps.  The number of microbes, predators, microbe respiration, predator respiration, and average enzymes, detritus, and food were stored in lists at each time step.  After 500 time steps, the oldest entries were discarded from the lists and the newest added, always ensuring that the latest 500 time steps were retained.  The average of these lists were then reported at the end of each simulation.  

## 5. Initialization

The model is initialized by setting up the cells in the environment, initializing microbes and predators, and if part of the requested simulation, roots in the environment.  Each is described below, with pseudo-code given for the sub-model that grows roots.

To setup cells in the environment, an initial quantity of detritus is assigned, if roots are not being simulated (in that instance, detritus is initialized in a following step).  Food, enzymes, and the root indicator are set to zero.  The color used to display the cell is initialized based on detritus present.

To setup microbes, a number of microbes are created based on an adjustable global variable.  Their appearance is set, energy is initialized to a global variable describing initial microbe energy, and their location in the environment is set randomly.  Two accumulators showing the energy consumed and energy respired are set to zero. 

Predators are initialized in a manner analogous to microbes.  The number of predators to create is controlled by their own global variable (i.e., initial predators), and the energy they begin with is set by a global parameter, initial predator energy.

Roots are grown within the environment to initialize the system.  In summary, a number of root tips are created, and then those tips grow in response to a series of parameters.  As the root tips move through the environment, the cells they move through are labeled as roots.   In the current application, roots grow only during initialization; during simulations, root patches do not change.  In pseudo-code, the method is:

	Create a number of root tips, controlled by a global variable, and place them 				randomly 
	Assign to the root tips a random heading
	While root length, represented by the sum of cells that are roots, 
	   	is less than an adjustable total root length 
		For one randomly selected root tip
			Turn the heading of the tip by an adjustable angle, 
				centered on zero, meaning no turn
			Move forward one cell and label it as root
			Initialize detritus in that cell, based on a global setting
			At some small probability
				Create a new root tip, representing a bifurcation of the
					root
				Set the heading of new tip based on a normally
					distributed mean and standard deviation angle
		Update the root length
	Continue while root length loop

The root tips that are created grow one cell at a time, each tip selected randomly for its oppor-tunity to grow.  This allows the initialization to match the number of cells that should be root precisely.  A biased random walk is represented for the root tip.   As roots grow, there is a probability the tip will branch, creating a new root tip.   When that branching occurs, the heading of the new root tip is determined from a normal distribution with a specified mean and standard deviation.

With roots fully grown, the parameters controlling their initialization are no longer used, and roots are static during a simulation.  The remaining step for initialization is for the system to calculate root clumpiness (see section 7).  That clumpiness index indicates the linearity or contagion of roots.

## 6. Input data

No input data are used in the model.  The dynamics in the simulation derive from the model structure, parameters provided, and spatial heterogeneity.

## 7. Submodels

Here we expand upon the submodels cited in section 3 and others, and the logic and assess-ment of their structures.  Each is identified by the name used in simulation and their intent, as in section 3.  Pseudocode summarizes the submodel, then notes are provided.  For brevity we cite here that in every case entities are asked to perform a function, they do so asynchronously and with their orders determined randomly each time step.  The values of parameters used in the model are discussed in more depth in the manuscript accompanying this ODD.

### move 	 			(Microbes move)				

	For each microbe:
		Turn a random angle, to face a random direction
		Move forward equal to the value of the microbial speed variable
		Subtract microbial movement energy cost from the microbe’s energy
	Next microbe

A random walk model was adopted for microbes as the most parsimonious approach to repre-senting microbial movements, and is a common approach (Duffy 1995).  The bacteria repre-sented by microbe agents are mobile but passive, changing location as they are carried along as ecosystem elements diffuse.

### move-predators 		(Predators move)					

Predators were simulated in a manner analogous to microbes, and follow the pseudocode shown under ‘move.’  Predators have their own speed and energy costs for movement.  Future applications may include sensing and hunting behavior in predators, but for simplicity this application uses random walk movements.

### produce-enzyme  		(Microbes produce enzymes)

	For each microbe:
		If the quantity of enzyme in the cell is less than a threshold and 
			If the microbe has more energy than the cost of producing an
				enzyme times a buffer then
			Add an enzyme unit to the cell enzyme quantity
			Reduce microbe energy by the cost of producing an enzyme
		End if 
	Next microbe

A threshold is compared to the enzyme concentration in the cell, allowing a microbe on the cell to avoid producing enzyme if it would only add to an already abundant concentration.  There is also a buffer internal to the microbe that dictates that the organisms’ energy must be above some threshold prior to enzyme production.  This allowed us to capture the idea that microbes may not produce enzymes if they are extremely low in energy.

### diffuse-enzyme		(Diffuse enzymes across the soil)

This is a single-line sub-model and so pseudocode is not needed.  The sub-model diffuses enzymes across the cells in the model.  The structure of that process is that a diffusion rate between 0.0 and 1.0 is provided, and that value influences the flow of quantities from each focal cell to its eight neighbors.  If the value is 0.0, no diffusion occurs.  If the value is 1.0, the contents of the cell flow completely into the neighboring cells, with each receiving 12.5% of the contents (8 cells x 12.5 = 100%).  That said, material in neighboring cells may diffuse back into the focal cell.  Most processes in the model are asynchronous, but diffusion is syn-chronous.  In this context, that means that a quantity that flows from cell A to cell B does not then flow back from B into A during the same time-step.  All flows are calculated for each cell in turn, and then quantities in the soil as a whole are updated. 

### convert-to-food		(Convert detritus to food)

	For each cell:	
	    If detritus is present and enzyme is present then
	       The number of active enzymes is the enzymes present times an enzyme
                   efficiency times the maximum detritus that can be converted divided
                   by beta plus the amount of detritus present 
            For each active enzyme and while food is available:
	       For the number of food units that an enzyme can convert:
	           	Increase food in the cell by one food unit
	  		Decrease detritus in the cell by one food unit
				If detritus is negative, set to zero
				Next food unit
			Next active enzyme
	Next cell

This sub-model incorporates a commonly used model of the limitation of enzymatic activity on detritus.  Enzymes are able to convert a number of detritus units into food units that may be adjusted prior to a simulation.  

### feed				(Microbes feed)

	For each microbe:
            Determine the food present, up to an adjustable maximum quantity of food
	    Calculate the energy of that food quantity
	    Increase energy of the microbe by the energy in the food, minus respiration
	    Decrease the food in the cell by the amount eaten
	    Increase an accumulator by the energy acquired
	    Increase an accumulator by the energy that is respired
	Next microbe

The amount of food that a microbe may consume is constrained to a maximum amount.  This reflects the handling time the microbe may have of food particles.  Respiration is a typical representation, where some proportion of energy acquired is lost to metabolic processes.  Ac-cumulators are used to store the sum of quantities through the duration of the simulation, and are not involved in ecosystem processes.

### feed-predators 		(Predators feed)					

	For each predator:
		Determine the number of microbes in the cell the predator occupies
		If there are microbes present then
 		    Set the number of microbes to eat if it exceeds a maximum number of 
		    prey by randomly selecting the maximum number from those present
			Increase energy of the predator by the accumulated energy of the
                             microbes consumed, minus a portion used in respiration
			Increase an accumulator storing the energy consumed and stored
			Increase an accumulator storing the energy that was respired
			Remove the microbes that were eaten
		End if
	Next predator

The number of microbes that predators may eat is limited, reflecting the handling time of the prey by the predator.  Predators accumulate the energy that is in the microbes they consume, so their feeding on ‘healthy’ microbes will yield more energy than feeding on emaciated mi-crobes.  As elsewhere, accumulators store sums during each simulation, in case those quantities are of interest.  They do not play a role in ecosystem processes.

### mortality			(Mortality of microbes, energy depleted)

	For each microbe:
		If the energy of the microbe is less than zero then
		    Contribute some material to detritus representing the microbe carcass
		    Kill the microbe
		End if
	Next microbe

If a microbe depletes its energy, which occurs when it moves about but cannot locate food, the microbe dies.  This causes a small quantity to be added to detritus of the cell that was oc-cupied, representing the cell walls, etc. of the microbe.

### mortality-predators 		(Mortality of predators, energy depleted)

Mortality in predators is represented analogously to that in microbes, and so pseudo-code is not provided.  Predators have their own variable representing the quantity of detritus their death causes to be added to the detritus pool of the cell.

### reproduce			(Reproduce microbes)

	For each microbe:
		If the microbe has more energy than the threshold for reproduction then
			Divide the energy of the microbe by two
			Create a new microbe
		End if
	Next microbe

Reproduction in the microbes is represented in a straightforward way.  If the microbe has suf-ficient energy, it fissions into two identical microbes, each with half the energy of the original microbe. The division of energy in two prior to creating a new microbe reflects the method used in the modeling platform.  When a new individual is created by another individual (it’s parent), that individual inherits the properties of the parent.  Therefore, by setting the energy of a ‘parent’ to one-half its current energy, then having that parent spawn an offspring that has an equal amount of energy, we conclude with two identical organisms with the total energy divided between them.

### reproduce-predators	(Reproduce predators)

Predators reproduce in a manner analogous to microbes, and so pseudo-code is not shown.  Predators have their own global variable representing the energy required to divide.

### decay-enzymes		(Allow enzymes to decay)	

	Calculate decay rate of enzymes as 0.50 divided by a parameter controlling
              half-life
	For each cell:
		Increase an accumulator of enzymes that were lost by the enzyme present -
		   times the decay rate
		Decrease enzyme in the cell by its quantity times 1 minus the decay rate
	Next cell

Enzymatic decay is represented in a straightforward way, once the decay rate is calculated.  In practice , a decay-half-life of 10 yields a rate of 0.5 / 10 or a decay rate  of  0.05.  A very short half-life such as 1 yields 0.5 / 1 and a rate of 0.5, and a very long half-life such as 1000 yields 0.5 / 1000 and a rate of 0.0005.

### detritus-rain			(Rain detritus continuously)

	Calculate the detritus added per patch, which in this case is the amount of
             detritus to be added per time step divided by the number of cells to
             receive detritus
	For a randomly selected number of cells indicated by a global variable:
		Increase detritus by the detritus to be added per cell
		Increment an accumulator storing the total detritus added to the system
	Next cell

This sub-model represents continuous detritus contributions to a subset of cells that are select-ed randomly each time step.  That number of cells is controlled by a global variable.  

### detritus-period		(Rain detritus periodically)

	Calculate the detritus added per patch, which in this case is the amount of
           detritus to be added per time step times the number of time steps defining
           the period divided by the number of cells to receive detritus
	For a randomly selected number of cells indicated by a global variable:
		Increase detritus by the detritus to be added per cells
		Increment an accumulator storing the total detritus added to the system
	Next cell

This sub-model represents periodic detritus contributions to a subset of cells that are selected randomly each time step.  That number of cells is controlled by a global variable.  The simula-tion uses the amount of detritus multiplied by the period to judge how much detritus should be added at the beginning of the period.

### detritus-root-continuous	(Detritus from roots, continuously)

	Calculate the detritus added per patch, which in this case is the amount of
            detritus to be added per time step divided by the number of root cells to
            receive detritus
	If a cell is a root then
		Increase detritus by the detritus to be added per cell
		Increment an accumulator storing the total detritus added to the system
	End if

This sub-model represents continuous detritus contributions to the cells that are defined as roots.  Cells do not change their status as roots during a simulation.

### detritus-root-periodic	(Detritus from roots, periodic)
  
	Calculate the detritus added per patch, which in this case is the amount of
           detritus to be added per time step times the number of time steps defining
           the period divided by the number of root cells receiving detritus
	If a cell is a root then
		Increase detritus by the detritus to be added per cells
		Increment an accumulator storing the total detritus added to the system
	End if

This sub-model represents periodic detritus contributions to a subset of cells that are defined as roots.  The simulation uses the amount of detritus multiplied by the period to judge how much detritus should be added at the beginning each period.  The status of cells as root or non-root does not change during a simulation in the current model.

### get-clumpiness		(Calculate clumpiness index for roots)

	Calculate the proportion of the cells in roots; call the result P1
	For each cell that is a root:
		For the four neighbors of the cell:
			If the neighbor is a root, increment root touching root by one
			If the neighbor is not a root, increment root touching non-root
			    by one
		Next neighbor
	Next cell

	Calculate as the number of root cells as neighbors divided by the number of -	
	   root cells with roots or non-roots as neighbors; call the result G1
	If G1 is greater than or equal to P1, calculate clumpiness as G1 minus P1 - 
	   divided by 1 minus P1
	If G1 is less than P1 and P1 is greater than 0.5, calculate clumpiness as G1
           minus P1 - 
	   divided by 1 minus P1
	If G1 is less than P1 and P1 is less than 0.5, calculate clumpiness as P1 minus
            G1 - 
	   divided by -1 times P1

This sub-model calculates the class-level clumpiness index found in McGarigal et al. (2012), with roots as the focal class.  If roots were distributed more regularly than random, which is unlikely, the clumpiness metric would approach -1.  If the distribution of roots are random, the index would be about 0.  If there is clustering of the roots, the index approaches 1.

### draw-patches			(Update cell colors displayed for the environment)

	Calculate mean values for detritus, food, and enzymes within the cells
	Calculate standard deviations for detritus, food, and enzymes within the cells
	Calculate the range (maximum minus minimum) for detritus, food, and enzymes
	Calculate a stretching factor for detritus, food, and enzymes such that -
	   each is defined by the mean minus twice the standard deviation
	For each cell
		Calculate a color value for detritus, food, and enzymes based on -
		   value in cell minus the factor calculated divided by the range times -
		   the desired range
		Trim the result to be between zero and the desired range
		Paint the display using the color values calculated
	Next cell

This sub-model is not used to represent soil biology, but rather updates the simulation display.  An element on the simulation interface allows users to choose to show individual results for one of detritus, food, or enzymes, or a composite image.  In the composite, detritus amounts are shown in red, food in green, and enzymes in blue.   The algorithm described above is a typical stretching used by popular geographic software, using twice the standard deviation to identify low and high values of the linear stretch.  


## Literature Cited

Duffy, K.J., P.T. Cummings, and R.M. Ford.  1995.  Random walk calculations for bacterial migration in porous media.  Biophysical Journal 68:800-806.

McGarigal, K., SA Cushman, and E Ene. 2012. FRAGSTATS v4: Spatial Pattern Analysis Program for Categorical and Continuous Maps. Computer software program produced by the authors at the University of Massachusetts, Amherst. Available at the following web site: http://www.umass.edu/landeco/research/fragstats/fragstats.html

### This model is to support: 

Moore, et al.  In review.  Enyzmatic and detrital influences on the structure, function, and dynamics of spatially-explicit model ecosystems.   Biochemistry

August 28, 2013
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
0
Rectangle -7500403 true true 151 225 180 285
Rectangle -7500403 true true 47 225 75 285
Rectangle -7500403 true true 15 75 210 225
Circle -7500403 true true 135 75 150
Circle -16777216 true false 165 76 116

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 5.0.4
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
<experiments>
  <experiment name="root-experiment-Nov-10-Run" repetitions="30" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <timeLimit steps="10000"/>
    <exitCondition>count predators = 0 or count microbes = 0</exitCondition>
    <metric>count microbes</metric>
    <metric>count predators</metric>
    <metric>clumpiness</metric>
    <metric>mean recent-microbes</metric>
    <metric>standard-deviation recent-microbes</metric>
    <metric>mean recent-predators</metric>
    <metric>standard-deviation recent-predators</metric>
    <metric>mean recent-microbe-respired</metric>
    <metric>standard-deviation recent-microbe-respired</metric>
    <metric>mean recent-predator-respired</metric>
    <metric>standard-deviation recent-predator-respired</metric>
    <metric>mean recent-enzyme</metric>
    <metric>standard-deviation recent-enzyme</metric>
    <metric>mean recent-detritus</metric>
    <metric>standard-deviation recent-detritus</metric>
    <metric>mean recent-food</metric>
    <metric>standard-deviation recent-food</metric>
    <steppedValueSet variable="detritus-per-tick" first="0" step="500" last="5000"/>
    <enumeratedValueSet variable="decay-half-life">
      <value value="1"/>
      <value value="10"/>
      <value value="100"/>
      <value value="1000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Detritus-model">
      <value value="&quot;Root, Continuous&quot;"/>
      <value value="&quot;Root, Periodic&quot;"/>
      <value value="&quot;Rain, Continuous&quot;"/>
      <value value="&quot;Rain, Periodic&quot;"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="Predators-Jan-25-2013" repetitions="10" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <timeLimit steps="10000"/>
    <exitCondition>count predators = 0 or count microbes = 0</exitCondition>
    <metric>count microbes</metric>
    <metric>count predators</metric>
    <metric>clumpiness</metric>
    <metric>mean recent-microbes</metric>
    <metric>standard-deviation recent-microbes</metric>
    <metric>mean recent-predators</metric>
    <metric>standard-deviation recent-predators</metric>
    <metric>mean recent-microbe-respired</metric>
    <metric>standard-deviation recent-microbe-respired</metric>
    <metric>mean recent-predator-respired</metric>
    <metric>standard-deviation recent-predator-respired</metric>
    <metric>mean recent-enzyme</metric>
    <metric>standard-deviation recent-enzyme</metric>
    <metric>mean recent-detritus</metric>
    <metric>standard-deviation recent-detritus</metric>
    <metric>mean recent-food</metric>
    <metric>standard-deviation recent-food</metric>
    <metric>total-microbes</metric>
    <metric>total-microbes-respired</metric>
    <metric>total-predators</metric>
    <metric>total-predators-respired</metric>
    <steppedValueSet variable="detritus-per-tick" first="0" step="500" last="5000"/>
    <enumeratedValueSet variable="decay-half-life">
      <value value="1"/>
      <value value="10"/>
      <value value="100"/>
      <value value="1000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Detritus-model">
      <value value="&quot;Root, Continuous&quot;"/>
      <value value="&quot;Root, Periodic&quot;"/>
      <value value="&quot;Rain, Continuous&quot;"/>
      <value value="&quot;Rain, Periodic&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="init-predators">
      <value value="80"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="NO-Predators-Jan-25-2013" repetitions="10" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <timeLimit steps="10000"/>
    <exitCondition>count microbes = 0</exitCondition>
    <metric>count microbes</metric>
    <metric>count predators</metric>
    <metric>clumpiness</metric>
    <metric>mean recent-microbes</metric>
    <metric>standard-deviation recent-microbes</metric>
    <metric>mean recent-predators</metric>
    <metric>standard-deviation recent-predators</metric>
    <metric>mean recent-microbe-respired</metric>
    <metric>standard-deviation recent-microbe-respired</metric>
    <metric>mean recent-predator-respired</metric>
    <metric>standard-deviation recent-predator-respired</metric>
    <metric>mean recent-enzyme</metric>
    <metric>standard-deviation recent-enzyme</metric>
    <metric>mean recent-detritus</metric>
    <metric>standard-deviation recent-detritus</metric>
    <metric>mean recent-food</metric>
    <metric>standard-deviation recent-food</metric>
    <metric>total-microbes</metric>
    <metric>total-microbes-respired</metric>
    <metric>total-predators</metric>
    <metric>total-predators-respired</metric>
    <steppedValueSet variable="detritus-per-tick" first="0" step="500" last="5000"/>
    <enumeratedValueSet variable="decay-half-life">
      <value value="1"/>
      <value value="10"/>
      <value value="100"/>
      <value value="1000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Detritus-model">
      <value value="&quot;Root, Continuous&quot;"/>
      <value value="&quot;Root, Periodic&quot;"/>
      <value value="&quot;Rain, Continuous&quot;"/>
      <value value="&quot;Rain, Periodic&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="init-predators">
      <value value="0"/>
    </enumeratedValueSet>
  </experiment>
</experiments>
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 1.0 0.0
0.0 1 1.0 0.0
0.2 0 1.0 0.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
0
@#$#@#$#@
